/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "xunfeispeechengine_p.h"
#include <unistd.h>
#include <cstring>
#include <curl/websockets.h>
#include "servererror.h"
#include "util.h"

bool XunfeiSpeechEnginePrivate::connectWebsocket(std::string hostUrl) {
    auto urlData = xunfei_speech_util::parseURL(hostUrl);
    std::string date = xunfei_speech_util::getGmtTime();
    std::string originSignature{"host: " + urlData["Host"] + "\n" +
                                "date: " + date + "\n" + "GET " +
                                urlData["Path"] + " HTTP/1.1"};
    std::string shaSignature = xunfei_speech_util::hmacSha256Encode(
        originSignature, secretKeyContinuous_);
    std::string authorization = "hmac username=\"" + apiKeyContinuous_ +
                                "\", algorithm=\"hmac-sha256\", headers=\"host "
                                "date request-line\", signature=\"" +
                                shaSignature + "\"";
    authorization = xunfei_speech_util::base64Encode(authorization);
    std::string callurl = hostUrl + "?" + "authorization=" +
                          xunfei_speech_util::urlEncode(authorization) +
                          "&date=" + xunfei_speech_util::urlEncode(date) +
                          "&host=" + urlData["Host"];

    CURLcode res;
    if (currentCurl_) {
        curl_easy_setopt(currentCurl_, CURLOPT_URL, callurl.c_str());
        curl_easy_setopt(currentCurl_, CURLOPT_CONNECT_ONLY, 2L);
        res = curl_easy_perform(currentCurl_);
        if (res == CURLE_HTTP_RETURNED_ERROR) {
            fprintf(stderr, "xunfei authenticate failed: %s\n",
                    curl_easy_strerror(res));
            currentError_ = ai_engine::lm::EngineError(
                ai_engine::lm::AiCapability::Speech,
                ai_engine::lm::EngineErrorCategory::Initialization,
                (int)ai_engine::lm::SpeechEngineErrorCode::Unauthorized,
                "接口鉴权失败");
            return false;
        }
        if (res != CURLE_OK) {
            fprintf(stderr, "net error: %s\n", curl_easy_strerror(res));
            currentError_ = ai_engine::lm::EngineError(
                ai_engine::lm::AiCapability::Speech,
                ai_engine::lm::EngineErrorCategory::Initialization,
                (int)ai_engine::lm::SpeechEngineErrorCode::NetworkDisconnected,
                curl_easy_strerror(res));
            return false;
        }
        return true;
    }
    currentError_ = ai_engine::lm::EngineError(
        ai_engine::lm::AiCapability::Speech,
        ai_engine::lm::EngineErrorCategory::Initialization,
        (int)ai_engine::lm::SpeechEngineErrorCode::UnsupportedScript,
        "创建资源失败");
    return false;
}

bool XunfeiSpeechEnginePrivate::connectRealtimeAsrWebsocket() {
    std::string timestampString = xunfei_speech_util::getCurrentUnixTimestamp();
    timestampString = appId_ + timestampString;
    timestampString = xunfei_speech_util::calculateMD5(timestampString);
    timestampString =
        xunfei_speech_util::hmacSha1Encode(timestampString, apiKeyRealTime_);
    std::string callurl =
        std::string(realtimeAsrHostUrl_) + "?" + "appid=" + appId_ + "&ts=" +
        xunfei_speech_util::urlEncode(
            xunfei_speech_util::getCurrentUnixTimestamp()) +
        "&signa=" + xunfei_speech_util::urlEncode(timestampString) +
        "&roleType=2" + "&engLangType=4";

    CURLcode res;
    if (currentCurl_) {
        curl_easy_setopt(currentCurl_, CURLOPT_URL, callurl.c_str());
        curl_easy_setopt(currentCurl_, CURLOPT_CONNECT_ONLY, 2L);
        res = curl_easy_perform(currentCurl_);
        if (res != CURLE_OK) {
            fprintf(stderr, "net error: %s\n", curl_easy_strerror(res));
            currentError_ = ai_engine::lm::EngineError(
                ai_engine::lm::AiCapability::Speech,
                ai_engine::lm::EngineErrorCategory::Initialization,
                (int)ai_engine::lm::SpeechEngineErrorCode::NetworkDisconnected,
                curl_easy_strerror(res));
            return false;
        }
        while (true) {
            size_t rlen;
            const curl_ws_frame *meta;
            char buffer[4100]{0};
            CURLcode curlCode = curl_ws_recv(currentCurl_, buffer,
                                             sizeof(buffer), &rlen, &meta);
            if (curlCode != CURLE_OK &&
                !(curlCode == CURLE_AGAIN || curlCode == CURLE_RECV_ERROR)) {
                fprintf(stderr, "net error: %s\n",
                        curl_easy_strerror(curlCode));
                currentError_ = ai_engine::lm::EngineError(
                    ai_engine::lm::AiCapability::Speech,
                    ai_engine::lm::EngineErrorCategory::Initialization,
                    (int)ai_engine::lm::SpeechEngineErrorCode::
                        NetworkDisconnected,
                    curl_easy_strerror(curlCode));
                return false;
            }
            if (!std::string(buffer).empty()) {
                if (int errorCode = xunfei_speech_server_error::parseErrorCode(
                        std::string(buffer))) {
                    fprintf(stderr, "xunfei authenticate failed %s.\n",
                            std::string(buffer).c_str());
                    currentError_ = ai_engine::lm::EngineError(
                        ai_engine::lm::AiCapability::Speech,
                        ai_engine::lm::EngineErrorCategory::Initialization,
                        (int)ai_engine::lm::SpeechEngineErrorCode::
                            NetworkDisconnected,
                        std::string(buffer));
                    return false;
                }
                return true;
            }
            usleep(1000);
        }
    }
    currentError_ = ai_engine::lm::EngineError(
        ai_engine::lm::AiCapability::Speech,
        ai_engine::lm::EngineErrorCategory::Initialization,
        (int)ai_engine::lm::SpeechEngineErrorCode::UnsupportedScript,
        "创建资源失败");
    return false;
}

void onWebSocket(bool (*doReceiveData)(XunfeiSpeechEnginePrivate *self),
                 XunfeiSpeechEnginePrivate *self) {
    while (true) {
        if (doReceiveData(self)) {
            usleep(1000);
            continue;
        }
        return;
    }
}

void XunfeiSpeechEnginePrivate::closeWebsocket() {
    size_t sent;
    if (currentCurl_) {
        curl_ws_send(currentCurl_, "", 0, &sent, 0, CURLWS_CLOSE);
    }
}

bool XunfeiSpeechEnginePrivate::doSendData(const char *data, int dataLength,
                                           int type) {
    size_t sent;
    CURLcode resultCode =
        curl_ws_send(currentCurl_, data, dataLength, &sent, 0, type);
    if (resultCode != CURLE_OK) {
        fprintf(stderr, "send data to xunfei failed: %s\n",
                curl_easy_strerror(resultCode));
        currentError_ = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech,
            ai_engine::lm::EngineErrorCategory::Initialization,
            (int)ai_engine::lm::SpeechEngineErrorCode::NetworkDisconnected,
            curl_easy_strerror(resultCode));
        return false;
    }
    return true;
}

bool XunfeiSpeechEnginePrivate::doReceiveOnceRecognitionData() {
    size_t rlen;
    const curl_ws_frame *meta;
    char buffer[4100]{0};

    CURLcode curlCode =
        curl_ws_recv(currentCurl_, buffer, sizeof(buffer), &rlen, &meta);
    if (curlCode == CURLE_AGAIN || curlCode == CURLE_RECV_ERROR) {
        return true;
    }
    if (curlCode != CURLE_OK) {
        fprintf(stderr, "net error: %s\n", curl_easy_strerror(curlCode));
        currentError_ = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech,
            ai_engine::lm::EngineErrorCategory::Initialization,
            (int)ai_engine::lm::SpeechEngineErrorCode::NetworkDisconnected,
            curl_easy_strerror(curlCode));
        runCallbackWithError(currentError_, OperationType::RecognizeOnce);
        return false;
    }

    if (int errorCode =
            xunfei_speech_server_error::parseErrorCode(std::string(buffer))) {
        fprintf(stderr, "xunfei recognize failed %s\n",
                std::string(buffer).c_str());
        auto errorTuple =
            xunfei_speech_server_error::asrErrorCode2speechResult(errorCode);
        currentError_ = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech, std::get<0>(errorTuple),
            (int)std::get<1>(errorTuple), std::string(buffer));
        runCallbackWithError(currentError_, OperationType::RecognizeOnce);
        return false;
    }

    std::string resultString = incompleteResult_.data() + std::string(buffer);
    if (resultString.substr(resultString.length() - 2) != "}}") {
        incompleteResult_ += std::string(buffer);
        return true;
    }
    incompleteResult_.clear();

    Json::Value recognitionJson =
        xunfei_speech_util::formatJsonFromString(resultString);

    ai_engine::lm::speech::RecognitionResult result;
    ai_engine::lm::EngineError error;
    result.error = error;
    result.type = ai_engine::lm::speech::ResultType::OnceRecognitionResult;
    result.speakerId = 0;
    for (int i = 0; i < recognitionJson["data"]["result"]["ws"].size(); i++) {
        for (int j = 0;
             j < recognitionJson["data"]["result"]["ws"][i]["cw"].size(); j++) {
            result.text +=
                recognitionJson["data"]["result"]["ws"][i]["cw"][j]["w"]
                    .asString();
        }
    }
    recognizingCallback_(result);
    recognizedCallback_(result);
    return false;
}

bool doReceiveContinuousRecognitionData(XunfeiSpeechEnginePrivate *self) {
    size_t rlen;
    const curl_ws_frame *meta;
    char buffer[4100]{0};

    if (self->isStopped_) {
        return false;
    }

    CURLcode curlCode =
        curl_ws_recv(self->currentCurl_, buffer, sizeof(buffer), &rlen, &meta);
    if (curlCode == CURLE_AGAIN || curlCode == CURLE_RECV_ERROR) {
        return true;
    }
    if (curlCode != CURLE_OK) {
        fprintf(stderr, "net error: %s\n", curl_easy_strerror(curlCode));
        self->currentError_ = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech,
            ai_engine::lm::EngineErrorCategory::Initialization,
            (int)ai_engine::lm::SpeechEngineErrorCode::NetworkDisconnected,
            curl_easy_strerror(curlCode));
        self->runCallbackWithError(
            self->currentError_,
            XunfeiSpeechEnginePrivate::OperationType::ContinuousRecognition);
        return false;
    }

    if (int errorCode =
            xunfei_speech_server_error::parseErrorCode(std::string(buffer))) {
        fprintf(stderr, "xunfei recognize failed %s\n",
                std::string(buffer).c_str());
        auto errorTuple =
            xunfei_speech_server_error::realtimeAsrErrorCode2speechResult(
                errorCode);
        self->currentError_ = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech, std::get<0>(errorTuple),
            (int)std::get<1>(errorTuple), std::string(buffer));
        self->runCallbackWithError(
            self->currentError_,
            XunfeiSpeechEnginePrivate::OperationType::ContinuousRecognition);
        return false;
    }

    std::string resultString =
        self->incompleteResult_.data() + std::string(buffer);
    Json::Value recognitionJson =
        xunfei_speech_util::formatJsonFromString(resultString);
    if (resultString[resultString.length() - 1] != '}' ||
        recognitionJson.isNull() || !recognitionJson.isObject() ||
        !recognitionJson.isMember("action") ||
        !recognitionJson.isMember("code") ||
        !recognitionJson.isMember("data") ||
        !recognitionJson.isMember("desc") || !recognitionJson.isMember("sid")) {
        self->incompleteResult_ += std::string(buffer);
        return true;
    }
    self->incompleteResult_.clear();

    Json::Value dataJson = xunfei_speech_util::formatJsonFromString(
        recognitionJson["data"].asString());

    if (dataJson["cn"]["st"]["type"].toStyledString() == "\"1\"\n") {
        return true;
    }

    if (self->lastSysthesisResult_.isNull()) {
        self->lastSysthesisResult_ = dataJson;
        return true;
    }

    ai_engine::lm::speech::RecognitionResult result;
    ai_engine::lm::EngineError error;
    result.error = error;
    result.type =
        ai_engine::lm::speech::ResultType::ContinuousRecognitionResult;
    std::string roleText;
    for (int k = 0; k < self->lastSysthesisResult_["cn"]["st"]["rt"].size();
         k++) {
        for (int i = 0;
             i < self->lastSysthesisResult_["cn"]["st"]["rt"][k]["ws"].size();
             i++) {
            for (int j = 0;
                 j <
                 self->lastSysthesisResult_["cn"]["st"]["rt"][k]["ws"][i]["cw"]
                     .size();
                 j++) {
                result.text += self->lastSysthesisResult_["cn"]["st"]["rt"][k]
                                                         ["ws"][i]["cw"][j]["w"]
                                                             .asString();
                roleText = self->lastSysthesisResult_["cn"]["st"]["rt"][k]["ws"]
                                                     [i]["cw"][j]["rl"]
                                                         .asString();
                int role = std::stoi(roleText);
                if (role != 0) {
                    self->currentRole_ = role;
                }
            }
        }
    }

    if (dataJson["cn"]["st"]["rt"][0]["ws"][0]["cw"][0]["wp"].asString() ==
        "p") {
        Json::Value lastWords;
        dataJson["cn"]["st"]["rt"][0]["ws"][0]["cw"].removeIndex(0, &lastWords);
        result.text += lastWords["w"].asString();
    }
    self->lastSysthesisResult_ = dataJson;
    result.speakerId = self->currentRole_;
    self->recognizedCallback_(result);
    return true;
}

bool doReceiveSynthesisData(XunfeiSpeechEnginePrivate *self) {
    size_t rlen;
    const curl_ws_frame *meta;
    char buffer[4100]{0};

    if (self->isStopped_) {
        return false;
    }

    CURLcode curlCode =
        curl_ws_recv(self->currentCurl_, buffer, sizeof(buffer), &rlen, &meta);
    if (curlCode == CURLE_AGAIN || curlCode == CURLE_RECV_ERROR) {
        return true;
    }
    if (curlCode != CURLE_OK) {
        fprintf(stderr, "net error: %s\n", curl_easy_strerror(curlCode));
        self->currentError_ = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech,
            ai_engine::lm::EngineErrorCategory::Initialization,
            (int)ai_engine::lm::SpeechEngineErrorCode::NetworkDisconnected,
            curl_easy_strerror(curlCode));
        self->runCallbackWithError(self->currentError_,
                                   self->currentSynthesisType_);
        return false;
    }

    if (int errorCode =
            xunfei_speech_server_error::parseErrorCode(std::string(buffer))) {
        fprintf(stderr, "xunfei synthesize failed %s\n",
                std::string(buffer).c_str());
        auto errorTuple =
            xunfei_speech_server_error::ttsErrorCode2speechResult(errorCode);
        self->currentError_ = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech, std::get<0>(errorTuple),
            (int)std::get<1>(errorTuple), std::string(buffer));
        self->runCallbackWithError(self->currentError_,
                                   self->currentSynthesisType_);
        return false;
    }

    std::string resultString =
        self->incompleteResult_.data() + std::string(buffer);
    if (resultString.substr(resultString.length() - 2) != "}}") {
        self->incompleteResult_ += std::string(buffer);
        return true;
    }
    self->incompleteResult_.clear();
    Json::Value synthesisJson =
        xunfei_speech_util::formatJsonFromString(resultString);

    ai_engine::lm::speech::SynthesisResult result;
    result.audioFormat = "mp3";
    result.audioRate = 16000;
    result.audioChannel = 1;
    auto dataString = xunfei_speech_util::base64Decode(
        synthesisJson["data"]["audio"].asCString());
    result.audioData = std::vector<char>{dataString.begin(), dataString.end()};
    ai_engine::lm::EngineError error;
    result.error = error;
    ai_engine::lm::speech::ResultType resultType;
    if (self->currentSynthesisType_ ==
        XunfeiSpeechEnginePrivate::OperationType::SynthesizeOnce) {
        resultType = ai_engine::lm::speech::ResultType::OnceSynthesisResult;
    }
    if (self->currentSynthesisType_ ==
        XunfeiSpeechEnginePrivate::OperationType::ContinuousSynthesis) {
        resultType =
            ai_engine::lm::speech::ResultType::ContinuousSynthesisResult;
    }
    result.type = resultType;
    self->synthesizingCallback_(result);

    if (synthesisJson["data"]["status"].asInt() == 2) {
        self->synthesizedCallback_(resultType);
        return false;
    }

    return true;
}

XunfeiSpeechEnginePrivate::XunfeiSpeechEnginePrivate() {}

XunfeiSpeechEnginePrivate::~XunfeiSpeechEnginePrivate() { releaseResources(); }

std::string XunfeiSpeechEnginePrivate::backendName() {
    return std::string("xunfei");
}

void XunfeiSpeechEnginePrivate::setConfig(const std::string &config) {
    Json::Value configJson = xunfei_speech_util::formatJsonFromString(config);
    if (configJson.isNull() || !configJson.isMember("apiKeyRealTime") ||
        !configJson.isMember("apiKeyContinuous") ||
        !configJson.isMember("secretKeyContinuous") ||
        !configJson.isMember("appId") ||
        !configJson["apiKeyRealTime"].isString() ||
        !configJson["apiKeyContinuous"].isString() ||
        !configJson["secretKeyContinuous"].isString() ||
        !configJson["appId"].isString()) {
        std::fprintf(stderr, "Invalid config for xunfei speech engine: %s\n",
                     config.c_str());
        return;
    }
    apiKeyRealTime_ = configJson["apiKeyRealTime"].asString();
    apiKeyContinuous_ = configJson["apiKeyContinuous"].asString();
    secretKeyContinuous_ = configJson["secretKeyContinuous"].asString();
    appId_ = configJson["appId"].asString();
}

void XunfeiSpeechEnginePrivate::setRecognizingCallback(
    ai_engine::lm::speech::RecognitionResultCallback callback) {
    recognizingCallback_ = callback;
}

void XunfeiSpeechEnginePrivate::setRecognizedCallback(
    ai_engine::lm::speech::RecognitionResultCallback callback) {
    recognizedCallback_ = callback;
}

void XunfeiSpeechEnginePrivate::setSynthesizingCallback(
    ai_engine::lm::speech::SynthesisResultCallback callback) {
    synthesizingCallback_ = callback;
}

void XunfeiSpeechEnginePrivate::setSynthesizedCallback(
    ai_engine::lm::speech::FinishedCallback callback) {
    synthesizedCallback_ = callback;
}

bool XunfeiSpeechEnginePrivate::initContinuousRecognitionModule(
    ai_engine::lm::EngineError &error) {
    currentError_ = error;
    if (!initResources(OperationType::ContinuousRecognition) ||
        !connectRealtimeAsrWebsocket()) {
        error = currentError_;
        return false;
    }
    closeWebsocket();
    releaseResources();
    return true;
}

bool XunfeiSpeechEnginePrivate::startContinuousRecognition(
    const std::string &params, ai_engine::lm::EngineError &error) {
    isStopped_ = false;
    currentError_ = error;

    if (!initResources(OperationType::ContinuousRecognition) ||
        !connectRealtimeAsrWebsocket()) {
        error = currentError_;
        return false;
    }
    setRecognitionParams_(params);

    currentThread_ =
        new std::thread(onWebSocket, doReceiveContinuousRecognitionData, this);
    currentThread_->detach();

    error = currentError_;
    return true;
}

bool XunfeiSpeechEnginePrivate::writeContinuousRecognitionAudioData(
    const std::vector<char> &audioData, ai_engine::lm::EngineError &error) {
    currentError_ = error;

    if (!recognizingCallback_ || !recognizedCallback_) {
        return false;
    }
    if (audioData.empty()) {
        error = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech,
            ai_engine::lm::EngineErrorCategory::Initialization,
            (int)ai_engine::lm::SpeechEngineErrorCode::TooLittleData,
            "音频数据为空");
        runCallbackWithError(error, OperationType::ContinuousRecognition);
        return false;
    }

    if (isStopped_) {
        return true;
    }

    if (!currentCurl_ || !currentThread_) {
        error = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech,
            ai_engine::lm::EngineErrorCategory::Initialization,
            (int)ai_engine::lm::SpeechEngineErrorCode::UnsupportedScript,
            "创建资源失败");
        runCallbackWithError(error, OperationType::ContinuousRecognition);
        return false;
    }

    if (audioData.size() > 1280) {
        auto datas = xunfei_speech_util::splitData(audioData, 1280);
        for (auto &data : datas) {
            doSendData(data.data(), data.size(), CURLWS_BINARY);
            usleep(40 * 1000);
        }
    } else {
        doSendData(audioData.data(), audioData.size(), CURLWS_BINARY);
    }

    if (currentStatus() == false) {
        runCallbackWithError(currentError_,
                             OperationType::ContinuousRecognition);
    }
    error = currentError_;
    return currentStatus();
}

bool XunfeiSpeechEnginePrivate::stopContinuousRecognition(
    ai_engine::lm::EngineError &error) {
    closeWebsocket();
    releaseResources();
    return true;
}

bool XunfeiSpeechEnginePrivate::destroyContinuousRecognitionModule(
    ai_engine::lm::EngineError &error) {
    closeWebsocket();
    releaseResources();
    return true;
}

bool XunfeiSpeechEnginePrivate::initRecognizeOnceModule(
    ai_engine::lm::EngineError &error) {
    currentError_ = error;
    if (!initResources(OperationType::RecognizeOnce) ||
        !connectWebsocket(asrHostUrl_)) {
        error = currentError_;
        return false;
    }
    closeWebsocket();
    releaseResources();
    return true;
}

bool XunfeiSpeechEnginePrivate::recognizeOnce(
    const std::string &params, const std::vector<char> &audioData,
    ai_engine::lm::EngineError &error) {
    isStopped_ = false;
    currentError_ = error;

    if (!recognizingCallback_ || !recognizedCallback_) {
        return false;
    }
    if (audioData.empty()) {
        error = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech,
            ai_engine::lm::EngineErrorCategory::Initialization,
            (int)ai_engine::lm::SpeechEngineErrorCode::TooLittleData,
            "音频数据为空");
        runCallbackWithError(error, OperationType::RecognizeOnce);
        return false;
    }

    setRecognitionParams_(params);
    Json::Value bodyJson;
    bodyJson["common"]["app_id"] = appId_;
    bodyJson["business"]["language"] = "zh_cn";
    bodyJson["business"]["domain"] = "iat";
    bodyJson["business"]["accent"] = "mandarin";
    bodyJson["data"]["status"] = 2;
    bodyJson["data"]["format"] = "audio/L16;rate=" + std::to_string(rate_);
    bodyJson["data"]["encoding"] = "raw";
    bodyJson["data"]["audio"] = xunfei_speech_util::base64Encode(
        std::string(audioData.data(), audioData.size()));
    if (!initResources(OperationType::RecognizeOnce) ||
        !connectWebsocket(asrHostUrl_)) {
        error = currentError_;
        runCallbackWithError(currentError_, OperationType::RecognizeOnce);
        return false;
    }
    if (!doSendData(bodyJson.toStyledString().c_str(),
                    bodyJson.toStyledString().size(), CURLWS_TEXT)) {
        error = currentError_;
        runCallbackWithError(currentError_, OperationType::RecognizeOnce);
        return false;
    }

    while (true) {
        if (doReceiveOnceRecognitionData()) {
            usleep(1000);
            continue;
        }
        closeWebsocket();
        error = currentError_;
        return currentStatus();
    }
}

bool XunfeiSpeechEnginePrivate::destroyRecognizeOnceModule(
    ai_engine::lm::EngineError &error) {
    closeWebsocket();
    releaseResources();
    return true;
}

bool XunfeiSpeechEnginePrivate::initContinuousSynthesisModule(
    ai_engine::lm::EngineError &error) {
    currentError_ = error;
    if (!initResources(OperationType::ContinuousSynthesis) ||
        !connectWebsocket(ttsHostUrl_)) {
        error = currentError_;
        return false;
    }
    closeWebsocket();
    releaseResources();
    return true;
}

bool XunfeiSpeechEnginePrivate::startContinuousSynthesis(
    const std::string &params, ai_engine::lm::EngineError &error) {
    isStopped_ = false;
    currentError_ = error;

    if (!initResources(OperationType::ContinuousSynthesis) ||
        !connectWebsocket(ttsHostUrl_)) {
        error = currentError_;
        return false;
    }

    setSynthesisParams_(params);

    currentSynthesisType_ = OperationType::ContinuousSynthesis;
    currentThread_ = new std::thread(onWebSocket, doReceiveSynthesisData, this);
    currentThread_->detach();

    error = currentError_;
    return true;
}

bool XunfeiSpeechEnginePrivate::writeContinuousSynthesisText(
    const std::string &text, ai_engine::lm::EngineError &error) {
    currentError_ = error;

    if (!synthesizingCallback_ || !synthesizedCallback_) {
        return false;
    }
    if (text.empty()) {
        error = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech,
            ai_engine::lm::EngineErrorCategory::Initialization,
            (int)ai_engine::lm::SpeechEngineErrorCode::TooLittleData,
            "文本为空");
        runCallbackWithError(error, OperationType::ContinuousSynthesis);
        return false;
    }

    if (isStopped_) {
        return true;
    }

    if (!currentCurl_ || !currentThread_) {
        error = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech,
            ai_engine::lm::EngineErrorCategory::Initialization,
            (int)ai_engine::lm::SpeechEngineErrorCode::UnsupportedScript,
            "创建资源失败");
        runCallbackWithError(error, OperationType::ContinuousSynthesis);
        return false;
    }
    Json::Value bodyJson;
    bodyJson["common"]["app_id"] = appId_;
    bodyJson["business"]["aue"] = "lame";
    bodyJson["business"]["sfl"] = 1;
    bodyJson["business"]["vcn"] = "xiaoyan";
    bodyJson["business"]["speed"] = speed_;
    bodyJson["business"]["volume"] = volume_;
    bodyJson["business"]["pitch"] = pitch_;
    bodyJson["business"]["tte"] = "UTF8";
    bodyJson["data"]["status"] = 2;
    bodyJson["data"]["text"] =
        xunfei_speech_util::base64Encode(std::string(text));

    doSendData(bodyJson.toStyledString().c_str(),
               bodyJson.toStyledString().length(), CURLWS_TEXT);

    if (currentStatus() == false) {
        runCallbackWithError(currentError_, OperationType::ContinuousSynthesis);
    }
    error = currentError_;
    return currentStatus();
}

bool XunfeiSpeechEnginePrivate::stopContinuousSynthesis(
    ai_engine::lm::EngineError &error) {
    closeWebsocket();
    releaseResources();
    return true;
}

bool XunfeiSpeechEnginePrivate::destroyContinuousSynthesisModule(
    ai_engine::lm::EngineError &error) {
    closeWebsocket();
    releaseResources();
    return true;
}

bool XunfeiSpeechEnginePrivate::initSynthesizeOnceModule(
    ai_engine::lm::EngineError &error) {
    currentError_ = error;
    if (!initResources(OperationType::SynthesizeOnce) ||
        !connectWebsocket(ttsHostUrl_)) {
        error = currentError_;
        return false;
    }
    closeWebsocket();
    releaseResources();
    return true;
}

bool XunfeiSpeechEnginePrivate::synthesizeOnce(
    const std::string &params, const std::string &text,
    ai_engine::lm::EngineError &error) {
    isStopped_ = false;
    currentError_ = error;

    if (!synthesizingCallback_ || !synthesizedCallback_) {
        return false;
    }
    if (text.empty()) {
        error = ai_engine::lm::EngineError(
            ai_engine::lm::AiCapability::Speech,
            ai_engine::lm::EngineErrorCategory::Initialization,
            (int)ai_engine::lm::SpeechEngineErrorCode::TooLittleData,
            "文本为空");
        runCallbackWithError(currentError_, OperationType::SynthesizeOnce);
        return false;
    }

    setSynthesisParams_(params);

    // std::string appId_ = "856f9ddd";
    Json::Value bodyJson;
    bodyJson["common"]["app_id"] = appId_;
    bodyJson["business"]["aue"] = "lame";
    bodyJson["business"]["sfl"] = 1;
    bodyJson["business"]["vcn"] = "xiaoyan";
    bodyJson["business"]["speed"] = speed_;
    bodyJson["business"]["volume"] = volume_;
    bodyJson["business"]["pitch"] = pitch_;
    bodyJson["business"]["tte"] = "UTF8";
    bodyJson["data"]["status"] = 2;
    bodyJson["data"]["text"] =
        xunfei_speech_util::base64Encode(std::string(text));

    if (!initResources(OperationType::SynthesizeOnce) ||
        !connectWebsocket(ttsHostUrl_)) {
        error = currentError_;
        runCallbackWithError(currentError_, OperationType::SynthesizeOnce);
        return false;
    }
    if (!doSendData(bodyJson.toStyledString().c_str(),
                    bodyJson.toStyledString().size(), CURLWS_TEXT)) {
        error = currentError_;
        runCallbackWithError(currentError_, OperationType::SynthesizeOnce);
        return false;
    }

    currentSynthesisType_ = OperationType::SynthesizeOnce;
    while (true) {
        if (doReceiveSynthesisData(this)) {
            usleep(1000);
            continue;
        }
        closeWebsocket();
        error = currentError_;
        return currentStatus();
    }
}

bool XunfeiSpeechEnginePrivate::destroySynthesizeOnceModule(
    ai_engine::lm::EngineError &error) {
    closeWebsocket();
    return true;
}

void XunfeiSpeechEnginePrivate::setRecognitionParams_(
    const std::string &params) {
    Json::Value paramsJson = xunfei_speech_util::formatJsonFromString(params);
    format_ = paramsJson["format"].asString();
    rate_ = paramsJson["rate"].asInt();
    channel_ = paramsJson["channel"].asInt();
    if (format_.empty()) {
        format_ = "pcm";
    }
    if (rate_ == 0) {
        rate_ = 16000;
    }
    if (channel_ == 0) {
        channel_ = 1;
    }
}

void XunfeiSpeechEnginePrivate::setSynthesisParams_(const std::string &params) {
    Json::Value paramsJson = xunfei_speech_util::formatJsonFromString(params);
    speed_ = paramsJson["speed"].asInt() * 10;
    volume_ = paramsJson["volume"].asInt() * 10;
    pitch_ = paramsJson["pitch"].asInt() * 10;
    if (speed_ == 0) {
        speed_ = 50;
    }
    if (volume_ == 0) {
        volume_ = 50;
    }
    if (pitch_ == 0) {
        pitch_ = 50;
    }
}

bool XunfeiSpeechEnginePrivate::initResources(OperationType type) {
    if (currentCurl_) {
        curl_easy_cleanup(currentCurl_);
        currentCurl_ = nullptr;
    }
    currentCurl_ = curl_easy_init();

    if (type == OperationType::RecognizeOnce ||
        type == OperationType::SynthesizeOnce) {
        return (currentCurl_ != nullptr);
    }

    if (currentThread_) {
        delete currentThread_;
        currentThread_ = nullptr;
    }

    return true;
}

void XunfeiSpeechEnginePrivate::releaseResources() {
    isStopped_ = true;
    if (currentCurl_) {
        curl_easy_cleanup(currentCurl_);
        currentCurl_ = nullptr;
    }
    if (currentThread_) {
        delete currentThread_;
        currentThread_ = nullptr;
    }
}

bool XunfeiSpeechEnginePrivate::currentStatus() {
    return (currentError_.getCode() == -1);
}

void XunfeiSpeechEnginePrivate::runCallbackWithError(
    const ai_engine::lm::EngineError &error, OperationType type) {
    if (type == OperationType::RecognizeOnce) {
        ai_engine::lm::speech::RecognitionResult result{
            std::string(), error,
            ai_engine::lm::speech::ResultType::OnceRecognitionResult, 0};
        recognizedCallback_(result);
    } else if (type == OperationType::ContinuousRecognition) {
        ai_engine::lm::speech::RecognitionResult result{
            std::string(), error,
            ai_engine::lm::speech::ResultType::ContinuousRecognitionResult, 0};
        recognizedCallback_(result);
    } else if (type == OperationType::SynthesizeOnce) {
        ai_engine::lm::speech::SynthesisResult result{
            std::string(),
            0,
            0,
            std::vector<char>(),
            error,
            ai_engine::lm::speech::ResultType::OnceSynthesisResult};
        synthesizingCallback_(result);
        synthesizedCallback_(
            ai_engine::lm::speech::ResultType::OnceSynthesisResult);
    } else if (type == OperationType::ContinuousSynthesis) {
        ai_engine::lm::speech::SynthesisResult result{
            std::string(),
            0,
            0,
            std::vector<char>(),
            error,
            ai_engine::lm::speech::ResultType::ContinuousSynthesisResult};
        synthesizingCallback_(result);
        synthesizedCallback_(
            ai_engine::lm::speech::ResultType::ContinuousSynthesisResult);
    }
}
